///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//         
// Filename: JTC_NTSC_GFX_DRV_v00x.h
//
// Author: JT Cook - http://www.avalondreams.com
// 
// Description: Support file for 240x192 tile/sprite driver (32x32 scrolling tile map - 30x24 visible)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// watch for multiple inclusions
#ifndef JTC_NTSC_GFX_DVR_v004
#define JTC_NTSC_GFX_DVR_v004

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Wait for Vsync macros
#define WAIT_FOR_VSYNC_START()  while(!g_VsyncFlag);
#define WAIT_FOR_VSYNC_END()	while(g_VsyncFlag);

#define MAX_SPRITES 8 //define max number of sprites
//define tile X,Y size
#define SCREEN_TILE_MAP_WIDTH 32
#define SCREEN_TILE_MAP_HEIGHT 32

// Types used for GFX_StartDrawing function (used primarily to setup the timer interrupt period
#define SCREEN_TYPE_NTSC		0
#define SCREEN_TYPE_VGA			1

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// STRUCTS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The following is the sprite structure
typedef struct{
	unsigned char X;					// X position of the sprite
	unsigned char Y;					// Y position of the sprite
	unsigned char *PixelData;		// Pointer to the bitmap data of the sprite
} Sprite, *SpritePtr;

//Scrolling map struct
typedef struct {
	unsigned char Y;   //Y position to scroll in
	unsigned char X;   //X position to scroll in
    unsigned char LastLine; //last line before we skip to next scroll values
    unsigned char dummy; //make sure we are alligned evenly
    unsigned char *MapAddress; //memory address where tile map is stored
} MapScroll, *MapScrollPtr;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

volatile extern unsigned int g_CurrentLine;   	// Current physical line 0...524(VGA),261(NTSC)
volatile extern int g_RasterLine;           	// Current logical raster line
volatile extern int g_VsyncFlag;				// Flag to indicate whether we are in v-sync or not

extern unsigned char *g_TileSet; 				// 16-bit pointer to a Character set as 8(6)x8 with 1 byte per pixel
extern Sprite *g_SpriteList;					// 16-bit pointer to sprite table records
extern MapScroll *g_MapScroll;                // 16-but point to map scrolling tale

extern int g_TileMapWidth;						// Used to store the tile map width for text console modes
extern int g_TileMapHeight;						// Used to store the tile map height for text console modes
extern int g_Tmap_Cursor_X;						// Text mode cursor x position
extern int g_Tmap_Cursor_Y;						// Text mode cursor y position

// Initializes the graphics engine
//void GFX_InitTile(unsigned char *TileData, unsigned char *TileMap, Sprite *SpriteTable);
void GFX_InitTile(unsigned char *TileData, Sprite *SpriteTable, MapScroll *MapXY);
// Starts the timer1 interrupt which calls our assembly video driver code
void GFX_StartDrawing(int ScreenType);

//fills the screen with a select character
void fill_screen(char digit, unsigned char *map_ptr, unsigned int map_size);

//places a tile
void draw_tile(int x, int y, char tile_pic, unsigned char *map_ptr);

// end multiple inclusions
#endif
