///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Micro Valley Racer
// Author: JT Cook - http://www.avalondreams.com
//
//
// Up/Down - gas/break
// left/right - steering
// Start - pause
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//#define PC_SIM 
#define XGS_CONSOLE 
//#define VGA
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef PC_SIM
#include "xgs_wrapper3.h" //for PC Dev
#include "bmp.h"
#endif

#ifdef XGS_CONSOLE
// Include main system header
#include "XGS_PIC_SYSTEM_V010.h"
#include "XGS_PIC_SOUND_DRV_V010.h" 

// Include graphics drivers
#include "JTC_NTSC_GFX_DRV_v006.h"
#include "JTC_NTSC_240_192_v006.h"

// Include the gamepad driver
#include "XGS_PIC_GAMEPAD_DRV_V010.h"
#endif 
#include "racer_gfx.h"
#include "font1bit.h"

#ifdef VGA
#include "ntsc_to_vga.h" //for NTSC to VGA color conversion
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RAND_RANGE(x,y) ( ( x ) + (rand()%( ( y ) - ( x )+1)))
#ifdef VGA
#else
#endif
#define left_lane 1
#define center_lane 7
#define right_lane 13 
#define road_width 14
#define min_road 9
#define max_road 16

#define max_com_cars 4 // how many computers cars that are in play at once
#define min_car_timer 30 //shortest amount of time for a car to respawn
#define max_car_timer 120 //180 //max amount of time for a car to respawn
#define max_crash_speed 2 //2 //difference in speed between computer car and player that would envoke a crash
#define min_oil 20 //smallest amount of time an oil patch will appear on the screen
#define max_oil 60 //largest amount of time an oil patch will appear on the screen
#define level_size 32*40 //how many tiles a level consists of until a new level is reached
#define start_clock 60
#define bonus_clock 20
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void main_loop();
int bounce_dir[8]; //direction for bouncing the sprites
float angle; //we will rotate the screen in a circle
Sprite g_Sprites[MAX_SPRITES];					// Max sprites
MapScroll g_Map[2];  //Map scrolling info
//graphics data
unsigned char g_SpriteData[256*6] __attribute__((far));
unsigned char g_TileData[64*(121)] __attribute__((far)); 
unsigned char game_state=0; //different game states (game loop, pause, game over, menu, etc)
unsigned char current_sound=0; //current sound that is playing
//--tiles
  //0-grass
  //1-left,left edge short
  //2-left, right edge long
  //3-left, left edge long
  //4-left, right edige short
  //5-road
  //6-right, left edge short
  //7-right, right edge long
  //8-right, left edge long
  //9-right, right edge short
  //10-checkered line
  //--
  //11-crashed car upper left
  //12-crashed car upper right
  //13-crashed car lower left
  //14-crashed car lower right
  //--
  //15-crashed car upper left
  //16-crashed car upper right
  //17-crashed car lower left
  //18-crashed car lower right
  //--
  //19-crashed car upper left
  //20-crashed car upper right
  //21-crashed car lower left
  //22-crashed car lower right
  //--
  //23-crashed car upper left
  //24-crashed car upper right
  //25-crashed car lower left
  //26-crashed car lower right
  //--
  //27-oil slick -main puddle
  //28-oil slick - right side
  //--
  //sand level
  //29-right, left edge short
  //30-right, right edge long
  //31-right, left edge long
  //32-right, right edge short
  //33-sand
  //34-water
  //--
  //snow level
  //35-grass
  //36-left,left edge short
  //37-left, right edge long
  //38-left, left edge long
  //39-left, right edige short
  //40-road
  //41-right, left edge short
  //42-right, right edge long
  //43-right, left edge long
  //44-right, right edge short
  // - tree
  //45,46,47,48,|49,50,51,52,|53,54,55,56,|57,58,59,60
  // - palm tree
  //61,62,63,64,|65,66,67,68,|69,70,71,72,|73,74,75,76
  // - snow tree
  //77,78,79,80,|81,82,83,84,|85,86,87,88,|89,90,91,92
  //LUT to to match colision with road
  //0-road, 1-edge of road, 2-grass/crash object, 3-oil slick
  const unsigned char Road_LUT[]={2,1,1,1,1,0,1,1,1,1,/*10*/0,2,2,2,2,2,2,2,2,2,/*20*/2,2,2,2,2,2,2,3,0,1,/*30*/1,1,1,1,2,
  /*35*/2,1,1,1,1,/*40*/0,1,1,1,1,1,2,2,2,2,/*50*/2,2,2,2,2,2,2,1,2,2,/*60*/2,2,2,2,2,2,2,2,2,2,/*70*/2,2,2,2,2,2,2,2,2,2,
  /*80*/2,2,2,2,2,2,2,2,2,2,/*90*/2,2,2};
  

//acceleration LUT
const unsigned char accel_LUT[]={1,2/* */,2,2/* */,4,4/* */,6,7/* */,9,10/* */,13,13/* */,17,17};
const unsigned char car_y_LUT[]={150,146,144,142,140,138,136,134,132,130,128,126,124,122};
//const unsigned char speed_LUT[]={1,2/* */,3,3/* */,4,4/* */,5,5/* */,6,6/* */,7,7/* */,8,8};
const unsigned char speed_LUT[]={0,1/* */,2,2/* */,3,3/* */,4,4/* */,5,5/* */,6,6/* */,7,7};
//LUT for when one car smashes against another car
const unsigned char hit_LUT[]={/*-0*/0,/*-1*/2,/*-2*/4,/*-3*/6,/*-4*/7,/*-5*/8,/*-6*/9};
//LUT for matching car speed with what speed to hit computer car with
const unsigned char speed_hit_LUT[]={/*-0*/2,/*-1*/3,/*-2*/3,/*-3*/3,/*-4*/3,/*-5*/3,/*-6*/3,/*7*/4,/*8*/4};


////////////////
unsigned int seed_counter; //seed the randomizer
unsigned char racer_animation; //frame for car
unsigned char racer_animation_timer; //timer to advance to next car frame
unsigned char max_racer_animation_timer; //max amount of time until racer animation advances
unsigned char car_speed; //how fast car is going, scrolling
unsigned char car_accel; //where car is at before accellerating to the next speed
unsigned char car_dir; //used for bouncing computer cars
unsigned char car_speed_scale; //used with LUT
unsigned char road_x, road_y, road_scroll, car_x, car_y;
unsigned char road_lane, road_dir, road_tile;
unsigned char road_switch; //timer used while creating a curve
unsigned char road_function; //what are we doing with the road (turning, going straight, etc)
                             // 0-straight, 1-turn left, 2-turn right
unsigned char car_shake;    //bool value to shake car when on edge of road
unsigned int oil_timer;     //timer to place new oil spot
unsigned int level_timer;   //timer until level is switched
unsigned char game_level;   //which stage player is on
unsigned int road_timer;    //when timer runs out, do a new type of road

unsigned char game_seconds; //game counter in seconds
unsigned char game_micro_sec; //micro seconds
unsigned char ready_go_value; //which are we displaying?
unsigned char ready_go_timer; //timer until we switch display
unsigned short int game_score; //player score
unsigned short int game_hiscore; //hi-score
unsigned short int engine_snd_timer=0; //timer for engine sound switching
unsigned char engine_sound=0; //toggle sound
// computer cars
typedef struct{
	unsigned char x; //x location of computer car
	unsigned char y; //y location of computer car
	unsigned char style; //what kind of car computer car is
	unsigned char speed; //speed of computer car
        unsigned char counter; // counter until new car will appear
        unsigned char turn; // a car needs to turn steer around a car
        unsigned char hit_level; //which power level the computer car is at for hit
        unsigned char hit_counter; //value we are counting until we goto next lower hit level
        unsigned char hit_dir; //which direction car is gliding when hit
	unsigned char hit_car; //car which first hit this car(while it is gliding)
} com_car_struct;

 com_car_struct com_car[max_com_cars];
//stuff on the side of the road
typedef struct{
	unsigned char x; //x starting location of tile
	unsigned char y; //used to count height of tile
	unsigned char object; //which object is currently being drawn
	unsigned char counter; //counter for when to draw a new side object
} road_object_struct;

road_object_struct road_object[2]; //2 one for left, one for right

// The TILE map buffer must be allocated into far memory space because 
// it is so large it wont fit into near 
unsigned char g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT] __attribute__((far));
unsigned char g_TileMap2[SCREEN_TILE_MAP_WIDTH*4] __attribute__((far));
unsigned char GamepadValue, GamepadValueOld;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void menu_setup();
void ready_set_go();
void menu();
void new_game();
void gameover_loop();
void do_gameover();
void game_loop();
void animate_racer();
void scroll_road();
void create_road(int tile_offset, char spacer);
void draw_crash(unsigned char car_number);
void crash_car();
unsigned char check_road_hit(unsigned char x, unsigned char y);
void new_car();
void new_comp_car(unsigned char car_number);
void check_comp_car(unsigned char number);
void move_comp_car(unsigned char car_number);
unsigned char check_sprite_hit(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2);
void copy_tile(char *source_tile, char *dest_tile);
void copy_sprite(char *source_sprite, char *dest_sprite);
void rotate_sprite_90(char *source_sprite, char *dest_sprite);
void rotate_sprite_180(char *source_sprite, char *dest_sprite);
void rotate_sprite_270(char *source_sprite, char *dest_sprite);
void get_game_chars();
void copy_font(int char_number, int char_data, unsigned char font_color, unsigned char font_bg);
void print_number(int x, int y, char *string);
void print_string(int x, int y, char *string);
void draw_timer();
void draw_hiscore();
void draw_score();
void draw_gameover();
void draw_hud();
void play_sound(unsigned char sound_number);
void sound_engine();

// Our main start location
#ifdef XGS_CONSOLE
int main (void)
#endif
#ifdef PC_SIM
void main_init ()
#endif
{
	int i;	

	// Always call ConfigureClock first
	SYS_ConfigureClock(FCY_NTSC);

	// Start up our display interrupt
	GFX_StartDrawing(SCREEN_TYPE_NTSC);

	// Init the gamepad
	Gamepad_Init();

	// Init the graphics system
    GFX_InitTile(g_TileData, g_Sprites, g_Map);
#ifdef VGA
    //convert sprites from NTSC to VGA color set
    for(i=0;i<256*5;i++)
     g_SpriteData[i]=ntsc_to_vga_lut[g_SpriteData[i]];

    //convert tiles from NTSC to VGA color set
    for(i=0;i<64*77;i++)
     g_TileData[i]=ntsc_to_vga_lut[g_TileData[i]];
#endif
     game_hiscore=0;
     menu_setup(); //setup menu screen
#ifdef XGS_CONSOLE
		// Initalize the sound driver
		SND_Init();
        main_loop();
	return(0);
#endif
}

//---------------------------------------------------------
//copy characters for menu
void menu_setup()
{
 #define f_color 0xAE
 #define f_bg 0x4E
 int n;
 SND_PlayTone(0); //turn off sounds
 fill_screen(0,g_TileMap,(32*32)); //clears the screen
 copy_font(0,0,f_color, f_bg); //copy font tiles
 print_string( 0, 1,"     +  * $ *$+ $$$+ *$$+");
 print_string( 0, 2,"     $+*$ $ $@  $  $ $ ?$");
 print_string( 0, 3,"     $?@$ $ $+  $$$@ $+ $");
 print_string( 0, 4,"     $  $ $ ?$@ $ ?+ ?$$@");

 print_string( 0, 6," $  $ *$$+ $   $   $$$ +   *");
 print_string( 0, 7," $  $ $  $ $   $   $+  ?+ *@");
 print_string( 0, 8," ?+*@ $$$$ $   $   $@   ?$@ ");
 print_string( 0, 9,"  ?@  $  $ $$$ $$$ $$$   $  ");   

 print_string( 0,11,"    $$$+ *$$+ *$+ $$$ $$$+");
 print_string( 0,12,"    $  $ $  $ $@  $+  $  $");
 print_string( 0,13,"    $$$@ $$$$ $+  $@  $$$@");
 print_string( 0,14,"    $ ?+ $  $ ?$@ $$$ $ ?+");

 print_string( 0,17,"          Hit start!");
 print_string(0,21,"     www.xgamestation.com");
 print_string(0,23,"JT Cook   www.avalondreams.com");

 //move sprites off screen
 for(n=0;n<8;n++)
     g_Sprites[n].X = 0;

 //setup screen
 g_Map[0].LastLine=250;
 g_Map[0].MapAddress=g_TileMap;
 g_Map[1].LastLine=251;
 g_Map[1].MapAddress=g_TileMap;  
 g_Map[0].X=0;
 g_Map[1].X=0;
 g_Map[0].Y=0;
 g_Map[1].Y=0;

 ready_go_value=0;
 ready_go_timer=60;
 //reset sound var
 current_sound=0;
}
//---------------------------------------------------------
//handle main menu
void menu()
{
		seed_counter++;
		GamepadValue = Gamepad_Read(0);		
		//Hit a button in the menu
		if(GamepadValue & GAMEPAD_ACTION1 | GamepadValue & GAMEPAD_ACTION2 | GamepadValue & GAMEPAD_START)
		{
 	        	GamepadValueOld = GamepadValue;//grab old key
			//seed randomizer
			srand(seed_counter);
                       	game_state=3;
			new_game();
                        
		}

}

//---------------------------------------------------------
//reset all the variables for game
void ready_set_go()
{
//  TIMEHSCORGAVNWDY 
// 01234567890123456
//           1111111
 unsigned const char s_ready[]={9,4,11,15,16,0};
 unsigned const char s_set[]={6,4,1,0};
 unsigned const char s_go[]={10,8,0};
 unsigned char n;
 unsigned short int offset;

 ready_go_timer++;
 if(ready_go_timer==40)
     SND_PlayTone(0);

 if(ready_go_timer>60)
 {
  ready_go_value++;
  ready_go_timer=0;
  //clear the line
  offset=(32*2)+13;
  for(n=0;n<7;n++)
    g_TileMap2[offset++]=94;
  switch(ready_go_value)
  {
   //draw ready
   case 1:
   {
    SND_PlayTone(1000);
    offset=(32*2)+13;
    for(n=0;n<9;n++)
    {
     if(s_ready[n]!=0)
       g_TileMap2[offset++]=s_ready[n]+94;
     else
      break;
    }
   }
   break;
   //draw set
   case 2:
   {
    SND_PlayTone(1000);
    offset=(32*2)+14;
    for(n=0;n<9;n++)
    {
     if(s_set[n]!=0)
       g_TileMap2[offset++]=s_set[n]+94;
     else
      break;
    }
   }
   break;
   //draw go
   case 3:
   {
	SND_PlayTone(2000);
    offset=(32*2)+14;
    for(n=0;n<9;n++)
    {
     if(s_go[n]!=0)
       g_TileMap2[offset++]=s_go[n]+94;
     else
      break;
    }
   }
   break;
   case 4:
    SND_PlayTone(0);
    game_state=1;
   break;
  }
 }
}

//---------------------------------------------------------
//reset all the variables for game
void new_game()
{
 int y,n;
	//seed randomizer -- debug
	srand(seed_counter);
        car_x=100;
        car_y=150;
	// setup sprite info
	g_Sprites[0].X = car_x;
	g_Sprites[0].Y = car_y;
	g_Sprites[0].PixelData = g_SpriteData;
	
        g_Sprites[1].X = 0;
	g_Sprites[1].Y =0;
        g_Sprites[2].X = 0;
	g_Sprites[2].Y =0;
        g_Sprites[3].X = 0;
	g_Sprites[3].Y =0;
        g_Sprites[4].X = 0;
	g_Sprites[4].Y =0;
        g_Sprites[5].X = 0;
	g_Sprites[5].Y =0;
        g_Sprites[6].X = 0;
	g_Sprites[6].Y =0;
        g_Sprites[7].X = 0;
        racer_animation=0;
        racer_animation_timer=0;
        max_racer_animation_timer=8;
        road_x=center_lane;
        road_lane=road_width;
	//road side objects
        road_object[0].object=0; //object type
        road_object[0].counter=4; //counter until new object
        road_object[0].y=0; //counter until new object
        road_object[1].object=0; //object type
        road_object[1].counter=9; //counter until new object
        road_object[1].y=0; //counter until new object

        game_level=0; //start at first level
	oil_timer=90; //placement of oil spot
	car_speed=0;
	car_dir=0;
        road_y=30;
        road_tile=0;
        road_timer=50;
	road_function=0;
        car_shake=0;
	car_speed_scale=0;

        //set up computer cars
        for(y=0;y<max_com_cars;y++)
        {
         com_car[y].counter=RAND_RANGE(min_car_timer,max_car_timer); //timer before we see a new car
         com_car[y].x=0;
         com_car[y].y=0;
         com_car[y].style=0; //no car 
         com_car[y].speed=0;
         com_car[y].turn=0;
         com_car[y].hit_level=0; 
         com_car[y].hit_counter=0; 
         com_car[y].hit_dir=0; 
	}

	//create road
        for(y=0;y<32;y++)
        {
          //create_road(((32-y)), (y&1));
          g_Map[1].Y-=8;
          road_y--;
          road_y&=0x1F;
          create_road(road_y,((y&1)^1));
        }

	level_timer=level_size;   //timer until level is switched

	//copy graphics data
	n=0;
	for(y=0;y<6;y++)
	{
	 copy_sprite(car_SpriteData+n,g_SpriteData+n);
	 n+=256;
	}

	n=0;
	for(y=0;y<93;y++)
	{
	 copy_tile(car_TileData+n,g_TileData+n);
	 n+=64;
	}

 //setup the map values
 g_Map[0].LastLine=24;
 g_Map[0].MapAddress=g_TileMap2;
 g_Map[1].LastLine=250;
 g_Map[1].MapAddress=g_TileMap;  
 g_Map[0].X=0;
 g_Map[1].X=0;
 g_Map[0].Y=0;
 g_Map[1].Y=0;
 get_game_chars();
 draw_hud();
 //game timer
 game_seconds=start_clock; //game counter in seconds
 game_micro_sec=60; //micro seconds
 game_score=0;
 draw_timer();
 draw_score();
 draw_hiscore();
}
//---------------------------------------------------------
void main_loop()
{
#ifdef XGS_CONSOLE
  while(1)
#endif
  {
// Gamestate
// 0 - menu
// 1 - in game
// 2 - game over
// 3 - ready set go
// 4 - pause

	switch(game_state)
	{
	 case 0:
	  menu();
	 break;
	 case 1:
	  game_loop();
	 break;
	 case 2:
		  SND_PlayTone(0); //turn off sounds
          WAIT_FOR_VSYNC_START();
          gameover_loop();
	      WAIT_FOR_VSYNC_END();
	 break;
	 case 3:
       WAIT_FOR_VSYNC_START();
	   ready_set_go();
	   WAIT_FOR_VSYNC_END();
	 break;
         case 4:
		 SND_PlayTone(0); //turn off sounds
           // Read the gamepad
	   GamepadValue = Gamepad_Read(0);
	   //UnPause the game
	   if((GamepadValue & GAMEPAD_START) && ((GamepadValueOld & GAMEPAD_START)==0))
           {
		  game_state=1;
           }
	   GamepadValueOld = GamepadValue;//grab old key
         break;
	}
  }
}
//---------------------------------------------------------
// just hang out for a few seconds
void gameover_loop()
{
         //handle game timer
         if(game_micro_sec>0)
           game_micro_sec--;
         else
	 { 
           game_micro_sec=60;
           if(game_seconds>0)
           {
	     game_seconds--;
           }
	   else
	   {
	    menu_setup();
            game_state=0;
	   }
	 }

}

//---------------------------------------------------------
// handle game over
void do_gameover()
{
 game_state=2;
 game_seconds=3;
 //if new hiscore, tack on a couple more victory seconds
 if(game_score==game_hiscore)
  game_seconds=5;
 game_micro_sec=60;
 draw_gameover();
}

//---------------------------------------------------------
// main loop for game
void game_loop()
{
  unsigned char n, hit;
	{
		// Wait until we are in v-sync to do any processing
		WAIT_FOR_VSYNC_START();
		if(racer_animation<0xFF)
		{
                 animate_racer();
		 // Read the gamepad
		 GamepadValue = Gamepad_Read(0);
		 // Adjust the sprites positions with gamepad up/down/left/right
                 //if the car hit an oil slick, then don't take direction
                 if(car_dir!=3)
                 {
		  if(GamepadValue & GAMEPAD_UP)
                  {
				if(car_speed_scale<13)
                                   car_accel++;
                                   if(car_accel>=accel_LUT[car_speed_scale])
                                   {
                                    car_accel-=accel_LUT[car_speed_scale];
                                    car_speed_scale++;
				    car_speed=speed_LUT[car_speed_scale];
				    car_y=car_y_LUT[car_speed_scale];                                
				    g_Sprites[0].Y=car_y;
                                   }
				//g_Sprites[0].Y--;
				//g_Map[0].Y--;
		  }
		  if(GamepadValue & GAMEPAD_DOWN)
                  {
				if(car_speed_scale>0)
                                {
				  car_speed_scale--;
				  car_speed=speed_LUT[car_speed_scale];			
				  car_y=car_y_LUT[car_speed_scale];
				  car_accel=0;
                                  g_Sprites[0].Y=car_y;
				}
				//g_Sprites[0].Y++;
				//g_Map[0].Y++;
		  }

		  car_dir=0;
		  if(GamepadValue & GAMEPAD_LEFT)
		  {
                                car_x-=car_speed;
				car_dir=1;
		  }
		  if(GamepadValue & GAMEPAD_RIGHT)
		  {
                                car_x+=car_speed;
				car_dir=2;
		  }
                 }
		 //if car has hit oil, count down the control lock
		 else
		 {
		  oil_timer--;
		  if(oil_timer<1)
                  {
		    car_dir=0; //no longer effected by the oil
		    oil_timer=RAND_RANGE(min_oil,max_oil);
		    racer_animation_timer=0;
                    racer_animation=0;
		    g_Sprites[0].PixelData = g_SpriteData;
		  }
		 }
		 //Pause the game
		 if((GamepadValue & GAMEPAD_START) && ((GamepadValueOld & GAMEPAD_START)==0))
                 {
		  game_state=4;
		 }
                 #ifdef PC_SIM //--bitmap function only on PC at this time
    	         if(GamepadValue & GAMEPAD_ACTION1)
		 {
                        //GFX_InitTile(g_TileData, &g_TileMap, g_Sprites, g_Map);
			draw_bmp_screen(g_TileData, g_Sprites, g_Map,"racer");
		 }
                 #endif
        	 GamepadValueOld = GamepadValue;//grab old key

        	 //shake car if on side of the road
        	 if(car_shake==1 && car_speed>0)
          	   g_Sprites[0].X+=2;
		 else
                   g_Sprites[0].X=car_x;			

        	 //check colision detection with car and road
        	 switch(check_road_hit(g_Sprites[0].X,g_Sprites[0].Y))
        	 {
         		case 0: //road
         		{
          		 car_shake=0;
         		}break;
         		case 1: //edge of road
         		{
          		 car_shake^=1;
         		}break;
         		case 2: //grass
         		{
          		 crash_car();
         		}break;
			case 3: //oil
			{
			 car_dir=3;
			 oil_timer=60; //the time the control will be locked for
			 racer_animation=0;
			}break;
        	 }

                 //check colision detection with computer cars and bounce them
                 for(n=0;n<max_com_cars;n++)
		 {
		          hit=check_sprite_hit(com_car[n].x, com_car[n].y,car_x, car_y);
			  if(hit==1)
			  {
                  	    com_car[n].hit_level=speed_hit_LUT[car_speed]; 
                  	    com_car[n].hit_counter=hit_LUT[com_car[n].hit_level];
		            com_car[n].hit_car=0xFF; //clear this player car hit it and not another computer car
			  }
			  //hit car moving left
                          if(hit==1 && car_dir==1)
			  {
			    //hit right side of car
                            com_car[n].hit_dir=1; 			    
			  }
			  //hit car moving right
			  else if(hit==1 && car_dir==2)
			  {
			    //hit the left side of a car
                            com_car[n].hit_dir=2; 
			  }
		 }

		} //after car has crashed, scroll screen past crash so we can't hit your own car again
		else if(racer_animation==0xFF)
		{
			if(car_y<180)
			{
			  car_speed=2;
			  car_y+=car_speed;
			}
			else
			{
			  racer_animation=0;
          		  new_car();
			}
		}	
	 //--handle computer cars
         for(n=0;n<max_com_cars;n++)
           move_comp_car(n);
        
	 //scroll screen
         scroll_road();

         //handle game timer
         if(game_micro_sec>0)
           game_micro_sec--;
         else
	 { 
           game_micro_sec=60;
           if(game_seconds>0)
           {
	     game_seconds--;
	     draw_timer();
           }
	   else
	    do_gameover();
	 }
	     sound_engine();		
         WAIT_FOR_VSYNC_END();
	}
	return 0;
}

//---------------------------------------------------------
//Handle a computer car
void move_comp_car(unsigned char car_number)
{
        unsigned char n, hit=0, hit2=0;
	unsigned char move_dir=0; //1-left, 2-right
        //put a new computer car in play 
	if(com_car[car_number].style==0)
        { 
          if(com_car[car_number].counter==0)      
	    new_comp_car(car_number);
          else
            com_car[car_number].counter--;
        }
        //computer car goes off screen
        else if(com_car[car_number].y>192+16) // || com_car[car_number].y<16)
        {
	  com_car[car_number].style=0;
	  com_car[car_number].x=0;
	  com_car[car_number].y=0;
          g_Sprites[car_number+1].X=0; //move computer car off screen
          com_car[car_number].counter=RAND_RANGE(min_car_timer,max_car_timer); //timer before we see a new car
        }
	//normal computer car moving
	else
	{
          com_car[car_number].y-=com_car[car_number].speed; //move computer car
          com_car[car_number].y+=car_speed; //adjust computer car with scrolling of screen

          //check to see if there are other cars ahead to drive around them
          for(n=0;n<max_com_cars;n++)
          {
           if(n!=car_number)
           {
            //check just ahead of car to see if we need to move computer car left or right
            hit=check_sprite_hit(com_car[car_number].x, com_car[car_number].y-32,com_car[n].x, com_car[n].y);
            hit2=check_sprite_hit(com_car[car_number].x, com_car[car_number].y-16,com_car[n].x, com_car[n].y);
            //if a car is infront of the computer car, try to avoid it
            if((hit==1 || hit2==1)&& com_car[car_number].speed>com_car[n].speed)
            {
              if(com_car[car_number].turn==0)
              {
               //check left side of the road to see there is road to drive on or not
	       if(  (check_road_hit(com_car[car_number].x-16,com_car[car_number].y)==0)
                  && (check_road_hit(com_car[car_number].x-32,com_car[car_number].y)==0) )
                {
                 com_car[car_number].turn=1;
                 //com_car[car_number].x-=com_car[car_number].speed;
		 move_dir=1;
                }
               else
		{
                 com_car[car_number].turn=2;
                 //com_car[car_number].x+=com_car[car_number].speed;
		 move_dir=2;
                }
/*
               //check left side of the road to see there is road to drive on or not
	       if(  (check_road_hit(com_car[car_number].x-16,com_car[car_number].y)==0)
                  && (check_road_hit(com_car[car_number].x-32,com_car[car_number].y)==0) )
                 com_car[car_number].x-=com_car[car_number].speed;
               else
                 com_car[car_number].x+=com_car[car_number].speed;
*/
              }
              else
              {
		 if(com_car[car_number].turn==1)
                   //com_car[car_number].x-=com_car[car_number].speed;
                   move_dir=1;
		 else
                   //com_car[car_number].x+=com_car[car_number].speed;
		   move_dir=2;
              }
              break; //since we match a car, no need to continue checking
            }
           //if there isn't a car we are driving around, put it back
           //else
           //   com_car[car_number].turn=0;

           }
          }
          //check to see if computer car needs to adjust so it doesn't run off road
          if(check_road_hit(com_car[car_number].x,com_car[car_number].y-16)!=0)
          {
           //check left side of car to see if there is road to drive on
           //car is not trying to get around computer car
//           if(com_car[car_number].turn==0)
           {
	    if(check_road_hit(com_car[car_number].x-16,com_car[car_number].y)!=0)
              //com_car[car_number].x+=com_car[car_number].speed;
              move_dir=2;
            else
              //com_car[car_number].x-=com_car[car_number].speed;
	      move_dir=1;
           }
/*
           //car is trying to get around computer car
           else
           {
	    if(check_road_hit(com_car[car_number].x-16,com_car[car_number].y)!=0)
            {
              com_car[car_number].x+=com_car[car_number].speed;
              com_car[car_number].turn==1;
            }
            else
            {
              com_car[car_number].x-=com_car[car_number].speed;
              com_car[car_number].turn==2;
            }
           }
*/
	  }
          
          //move car left and right
          if(com_car[car_number].hit_dir==0)
          {
	   switch(move_dir)
           {
            case 1: //left
	     com_car[car_number].x-=com_car[car_number].speed;
	     break;
            case 2: //right
	     com_car[car_number].x+=com_car[car_number].speed;
	     break;
           }
          }
          //or car is moving after getting hit
          else
          {
                  if(com_car[car_number].counter>0)
                   com_car[car_number].counter--;
                  else
                  {
		    if(com_car[car_number].hit_level>0)
                    {
		     com_car[car_number].hit_level--;
		     com_car[car_number].counter=hit_LUT[com_car[car_number].hit_level];           
		    }
		    else
 		     com_car[car_number].hit_dir=0;
			
		  }

	          switch(com_car[car_number].hit_dir)
           	  {
            	    case 1: //left
	     	     com_car[car_number].x-=com_car[car_number].hit_level;
	     	     break;
            	    case 2: //right
	             com_car[car_number].x+=com_car[car_number].hit_level;
	             break;
		  }
          }
    
          //check colision against other cars
          //if(com_car[car_number].hit_dir==0)
          {
	   hit=0;
	   for(n=0;n<4;n++)
           {
	    if(n!=car_number && n!=com_car[car_number].hit_car)
            {
		hit=check_sprite_hit(com_car[car_number].x, com_car[car_number].y,com_car[n].x, com_car[n].y);
                if(hit==1)
		{
		  com_car[n].hit_car=car_number; //tell it which car hit it
                  com_car[n].hit_level=3; 
                  com_car[n].hit_counter=hit_LUT[com_car[n].hit_level]; 
                  //since we hit another car, we will shift all the momentum to that car and stop
		  // the gliding from this one (if it was gliding)
		  com_car[car_number].hit_car=0xFF; //clear this
                  com_car[car_number].hit_level=0; 
                  com_car[car_number].hit_counter=0;
                  com_car[car_number].hit_dir=0;
		  play_sound(3); //hit car sound  		  
		  //we hit the left side of the car
		  if(com_car[car_number].x<com_car[n].x+8)
		  {
                    com_car[n].hit_dir=2; 
		  }
		  //we hit the right side of the car
		  else
		  {
                    com_car[n].hit_dir=1; 
		  }
		}
                   //printf("x1=%i,y1=%i,x2=%i,y2=%i\n",com_car[car_number].x, com_car[car_number].y,com_car[n].x, com_car[n].y);
             }
	   }

           //check colision detection with player car
           //{
		hit=check_sprite_hit(com_car[car_number].x, com_car[car_number].y,car_x, car_y);
                if(hit==1)
		{
			play_sound(3); //hit car sound
                  //com_car[n].hit_level=3; 
                  //com_car[n].hit_counter=hit_LUT[com_car[n].hit_level]; 
		  //crash the car if the player car hits a slow computer car, or slow it down                 
		  if((com_car[car_number].speed<car_speed) && (car_y>com_car[car_number].y+5))
		  {
		   //crash the car
		   if((car_speed-com_car[car_number].speed)>max_crash_speed)
                   {
		     crash_car();
		     //crash the computer car too		
		     draw_crash(car_number);
		   }
		   //slow the car down
		   else 
		   {
		     car_speed=com_car[car_number].speed-1;
		     car_accel=0;
		     //find the speed scale for the new speed
		     for(hit=0;hit<15;hit++)
	             {
		       if(speed_LUT[hit]==car_speed)
      		       {
		         car_speed_scale=hit;
			 car_y=car_y_LUT[car_speed_scale];
			 g_Sprites[0].Y=car_y;
			 break;
		       }
		     }
		   }
		  }
		  //we hit the left side of the car
		  else if((com_car[car_number].x+5)>car_x)
		  {
                    car_x-=4; 
		  }
		  //we hit the right side of the car
		  else
		  {
                    car_x+=4;
		  }
		}
	  //check colision detection against backgruond and crash
          if(check_road_hit(com_car[car_number].x,com_car[car_number].y)==2)
	  {
	       draw_crash(car_number);
	     
          }
          }

	  g_Sprites[car_number+1].X=com_car[car_number].x; 
	  //make sure we don't draw cars on the status bar
          if(com_car[car_number].y>(16+24))
	    g_Sprites[car_number+1].Y=com_car[car_number].y; 
	  else
	    g_Sprites[car_number+1].X=0; 
	}
}

//---------------------------------------------------------
//place a new computer car in play
void new_comp_car(unsigned char car_number)
{
  unsigned char n, road_length=0, road_x=0, car_x=0;
  unsigned int map_y;

  //make sure a newly introduced car is not going the same speed as the player car
  do
  {
  com_car[car_number].speed=RAND_RANGE(2,5);
  } while(com_car[car_number].speed==car_speed);

	
  //map_y=255-g_Map[0].Y;
  map_y=g_Map[1].Y-255;
  //if player car is going slower than speed of computer car, start computer car at bottom of screen
  if(car_speed<com_car[car_number].speed)
  {
    map_y+=191+16; //move to the bottom of the screen
    com_car[car_number].y=191+16;
  }
  else 
    com_car[car_number].y=16;
  map_y&=0xFF; //mask to screen limit
  map_y>>=3;
  map_y<<=5; //get map tile location
  map_y&=0x3FF; //make sure we don't go outside tile map
  //find where the road is and the width
  for(n=0;n<32;n++)
  {
   if(Road_LUT[g_TileMap[map_y+n]]==0 && road_length==0)
   {
    road_length++;
    road_x=n;
   }
   else if(Road_LUT[g_TileMap[map_y+n]]==0)
   {
    road_length++;
    //g_TileMap[map_y+n]=2; //debug
   }
  }
  //randomly select a location on the road
  car_x=RAND_RANGE(road_x,road_x+road_length-1);
  car_x<<=3;
  //setup new car
  com_car[car_number].x=car_x+16;

  //make sure car doesn't overlap with another car
  road_length=0; //reuse value for placing car
  for (n=0;n<max_com_cars;n++)
  {
   if(n!=car_number)
   {
     road_length=check_sprite_hit(com_car[car_number].x, com_car[car_number].y,com_car[n].x, com_car[n].y);
     if(road_length==1) // && com_car[n].style>0)
     {
       break;
     }
     else 
	road_length=0;
   }
  }
  //no overlapping car
  if(road_length==0)
  {
   com_car[car_number].style=RAND_RANGE(1,3);
   g_Sprites[car_number+1].PixelData = g_SpriteData+((com_car[car_number].style+1)<<8);
   //set up computer car sprites
   g_Sprites[car_number+1].X=com_car[car_number].x;
   //make sure we don't draw cars over status bar
   if(com_car[car_number].y>(16+24))
	g_Sprites[car_number+1].Y=com_car[car_number].y; 
   else
	g_Sprites[car_number+1].Y=0; 
  }
  //new car overlaps a current car, do not draw it
  else
  {
   g_Sprites[car_number+1].X=0;
   g_Sprites[car_number+1].Y=0;
   com_car[car_number].speed=0;
   com_car[car_number].y=0;
   com_car[car_number].x=0;
   com_car[car_number].style=0;
   com_car[car_number].counter=15; //reset timer
  }
  
  com_car[car_number].turn=0;
  com_car[car_number].hit_level=0; 
  com_car[car_number].hit_counter=0; 
  com_car[car_number].hit_dir=0; 
  com_car[car_number].hit_car=0xFF; //clear this
}

//---------------------------------------------------------
//check the colision detection of a sprite against another sprite
unsigned char check_sprite_hit(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
 #define s_len 13  //length of sprite we are checking
 //check Y axis 1st
 if(y1 <= y2+s_len && y1+s_len>=y2) 
 {
  //check X axis now
  if(x1 <= x2+s_len && x1+s_len>=x2) 
  {
   //printf("x=%i, y=%i, c_x=%i, c_y=%i\n",x1,y1,x2,y2);
   return(1); //hit
  }
 }
 return(0); //no hit
}

//---------------------------------------------------------
//check the colision detection of a sprite against the road
unsigned char check_road_hit(unsigned char x, unsigned char y)
{
  unsigned char x_left_value, x_right_value, y_car;
  unsigned short x_left, x_right;
  //compensate for sprite going off screen
  x-=16;
  y-=16-2;
  x_left=x+4; //pull edge inward more
  x_right=x+12; //pull edge inward
  x_left>>=3; //get tile location
  x_right>>=3; //get tile location
  //y-=g_Map[0].Y; //add scroll value
  y+=g_Map[1].Y; //add scroll value
  y_car=y>>3; //get tile location
  //get map location
  x_left+=(y_car<<5);
  x_right+=(y_car<<5);
  //--debug, write tiles to map to test values we are checking
  //g_TileMap[(x_left&0x3FF)]=4;
  //g_TileMap[(x_right&0x3FF)]=4;
  //get map values
  x_left_value=Road_LUT[g_TileMap[(x_left&0x3FF)]];
  x_right_value=Road_LUT[g_TileMap[(x_right&0x3FF)]];
  //crashable object  
  if(x_left_value==2 || x_right_value==2)
    return(2);
  //oil slick
  else if(x_left_value==3 || x_right_value==3)
    return (3);
  //side of road
  else if(x_left_value==1 || x_right_value==1)
    return (1);
  else 
    return(0);   
}

//---------------------------------------------------------
//draw the crashed car tile
void draw_crash(unsigned char car_number)
{
 unsigned int tile_offset;
 unsigned char y, x, crash_tile;
 #define player_number 11
 //get X, Y location
 crash_tile=player_number; //player car
 //player car
 if(car_number==player_number)
 {
  //y=(car_y-16)-g_Map[0].Y;
  y=(car_y-16)-(255-g_Map[1].Y);
  x=(car_x-16)>>3;
  g_Sprites[0].X=0; //hide car sprite
 }
 //computer car
 else
 {
  //y=(com_car[car_number].y-16)-g_Map[0].Y;
  y=(com_car[car_number].y-16)-(255-g_Map[1].Y);
  x=(com_car[car_number].x-16)>>3;
  g_Sprites[car_number+1].X=0; //hide car sprite
  crash_tile+=com_car[car_number].style<<2;
  com_car[car_number].style=0;
  com_car[car_number].counter=RAND_RANGE(min_car_timer,max_car_timer); //timer before we see a new car
 }
 y>>=3; //divide by 8
 //y<<=5; //multiply by 32 (32 tiles wide)
 tile_offset=((y<<5)+x)&0x3FF;
 //make sure if it is computer car and it crashes off screen, don't draw it
 if((com_car[car_number].y>25 && com_car[car_number].y<220)|| car_number==player_number)
 {
  g_TileMap[tile_offset]=crash_tile;
  g_TileMap[tile_offset+1]=crash_tile+1; 
  tile_offset+=32;
  tile_offset&=0x3FF;
  g_TileMap[tile_offset]=crash_tile+2;
  g_TileMap[tile_offset+1]=crash_tile+3; 
 }
 //remove car from playfield
 if(car_number!=player_number)
 {
    com_car[car_number].y=0;
	com_car[car_number].x=0;
 }
 play_sound(4); //crash sound
}

//---------------------------------------------------------
//handle the crash of the player car
void crash_car()
{
 car_speed=0;
 car_accel=0;
 car_dir=0;
 car_speed_scale=0;
 racer_animation=0xFF; //indicate a crash and to scrolls screen past crash
 draw_crash(11); //player car
}

//---------------------------------------------------------
// check road for computer cars to turn
void check_comp_car(unsigned char number)
{
  unsigned char n, road_x=0;
  unsigned int map_y;

  map_y=g_Map[1].Y;
  map_y+=com_car[number].y-16;

  map_y&=0xFF; //mask to screen limit
  map_y>>=3;
  map_y<<=5; //get map tile location
  map_y&=0x3FF; //make sure we don't go outside tile map
  //find where the road is and the width
  for(n=0;n<32;n++)
  {
   if(Road_LUT[g_TileMap[map_y+n]]==0)
   {
    road_x=n;
    break;
   }
  }
  road_x<<=3;
  //check to see where we move the computer car to
  if(road_x>com_car[number].x)
    com_car[number].x+=com_car[number].speed;
  else
    com_car[number].x-=com_car[number].speed;
}

//---------------------------------------------------------
//after crash, place a new car by the side of the road
void new_car()
{
  unsigned int tile_loc;
  unsigned char n, x=0, y=150;

  //y-=g_Map[0].Y; //add scroll value
  y+=g_Map[1].Y; //add scroll value
  y>>=3; //get tile location
  //find the road
  tile_loc=(y<<5);

  for(n=0;n<32;n++)
  {
   if(Road_LUT[g_TileMap[(tile_loc&0x3FF)]]!=0)
   {
    tile_loc++;
    x++;
   }
   else
   {
    break;
   }
  }
  x++;
  //get new car location
  car_x=x<<3;
  //car_x=150;
  g_Sprites[0].X=car_x;
  //--debug, write tiles to map to test values we are checking
  //g_TileMap[(tile_loc&0x3FF)]=4;
  car_speed=0;
  car_speed_scale=0;
  car_accel=0;
  car_y=car_y_LUT[car_speed_scale];
  g_Sprites[0].Y=car_y;
  racer_animation_timer=0;
  racer_animation=0;
  g_Sprites[0].PixelData = g_SpriteData;
}

//---------------------------------------------------------
//Handles scrolling the road, placing new tiles, etc
void scroll_road()
{
 g_Map[1].Y-=car_speed;
 road_scroll+=car_speed;

 if(road_scroll>7)
 {
  //new event?-------
  road_timer--;
  if(road_timer<1)
  {
   road_timer+=50;
   road_function=RAND_RANGE(0,6);
   if(road_function==1 && road_x==3)
      road_function=2;
   else if(road_function==2 && (road_x+road_lane)==27)
      road_function=1;
   if(road_function>0)
   {
      road_timer=(center_lane<<1);
      road_switch=0;

      //slow left
      if(road_function==3)
      {
       road_tile&=0xFE; //ready tile
       road_timer<<=1; //since it takes twice as long to move an tile, change the amount we are moving
       road_switch=1;
      }
      //slow right
      if(road_function==4)
      {
       road_tile|=0x01; //ready tile 
       road_timer<<=1; //since it takes twice as long to move an tile, change the amount we are moving
       road_switch=1;
      }
   }
  }
  //handle road functions----
  // 0-straight, 1-sharp left, 2-sharp right, 3-slow left, 4-slow right
  //go straight
  switch(road_function)
  {
   case 0:
   {
    road_tile^=1;
   }break;
   //left turn
   case 1:
   {
    if(road_tile&1)
     road_x--;
    if(road_x<2)
    {
     road_x++;
     road_function=0;
     road_timer=1;
     road_tile^=1;
    }
    road_tile^=1;
   }break;
   //right turn
   case 2:
   {
    if((road_tile&1)==0)
     road_x++;
    if(road_x+road_lane>27)
    {
     road_x--;
     road_function=0;
     road_timer=1;
     road_tile^=1;
    }
    road_tile^=1;
   }break;
   //slow left turn
   case 3:
   {
    if((road_switch&3)==0)
    {
     road_x--;     
    }
    if((road_switch&1)==0)
      road_tile^=1;

    if(road_x<2)
    {
     road_x++;
     road_function=0;
     road_timer=1;
     road_tile^=1;
    }
    road_switch++;
    road_switch&=0x3;
   } break;
   //slow right turn
   case 4:
   {
    if((road_switch&3)==0)
    {
     road_x++;     
    }
    if((road_switch&1)==0)
      road_tile^=1;

    if(road_x+road_lane>27)
    {
     road_x--;
     road_function=0;
     road_timer=1;
     road_tile^=1;
    }
    road_switch++;
    road_switch&=0x3;
   } break;
   //shrink road
   case 5:
   {
    road_tile^=1;
    if((road_tile&0x01)==0)
    {
        road_function=0;
        if(road_lane>min_road)
           road_lane--;
	else
	   road_function=6; //grow road since we hit our limit on size
    }
   } break;
   //grow road
   case 6:
   {
    road_tile^=1;
    if((road_tile&0x01)==1 && (road_x+road_lane)<27)
    {
	if(road_lane<max_road)
	{
           road_lane++;
        }
        road_function=0;
    }
    else if((road_tile&0x01)==0 && (road_x+road_lane+1)>27)
    {
        road_function=0;
	if(road_lane<max_road)
	{
           road_lane++;
	   road_x--;
        }
	else
	   road_function=5; //grow road since we hit our limit on size
    }

   } break;

  }
  road_scroll-=8;
  road_y--;
  road_y&=0x1F;
  game_score++;
  create_road(road_y,road_tile);
 }

}

//---------------------------------------------------------
//draws a horizontal strip of road
void create_road(int tile_offset, char spacer)
//draw the road background
{
 unsigned char x, object_offset;
 //since we are drawing a new strip of road, add to score
 draw_score();
 //find the current horizontal row we are working on
 tile_offset=(tile_offset&0x1F)<<5;
  {
   //left side of road grass
   for(x=0;x<road_x;x++)
     g_TileMap[tile_offset+x]=0;
   //left edge of road (made up of two tiles)
   g_TileMap[tile_offset+x]=1+(spacer<<1);
     x++;
   g_TileMap[tile_offset+x]=2+(spacer<<1);
     x++;
   //road
   for(x=x;x<(road_x+road_lane);x++)
     g_TileMap[tile_offset+x]=5;
   //right edge of road (made up of two tiles)
   //normal level
     if(game_level!=1)
     {
       g_TileMap[tile_offset+x]=6+(spacer<<1);
       x++;
       g_TileMap[tile_offset+x]=7+(spacer<<1);
       x++;
       for(x=x;x<31;x++)
          g_TileMap[tile_offset+x]=0; 
     }
     //beach level
     else if(game_level==1)
     {
       g_TileMap[tile_offset+x]=29+(spacer<<1);
       g_TileMap[tile_offset+x+1]=33;
       g_TileMap[tile_offset+x+2]=33;
       g_TileMap[tile_offset+x+3]=33;
       g_TileMap[tile_offset+x+4]=30+(spacer<<1);
       x+=5;
       for(x=x;x<31;x++)
          g_TileMap[tile_offset+x]=34;
     }
  }
 //handle the snow level
 if(game_level==2)
  for(x=0;x<31;x++)
    g_TileMap[tile_offset+x]+=35;
 //------------------------------
 //check road side object
 for(x=0;x<2;x++)
 {
  if(road_object[x].counter<1)
  {
   //new object, set it up
   if(road_object[x].object==0 && level_timer>5)
   {
    switch(game_level)
    {
     case 0: //grass level
      road_object[x].object=1; //tree
      break;
    case 1: //beach level
     road_object[x].object=2; //palm tree
     break;
    case 2: //snow level
     road_object[x].object=3; //snow tree
     break;
    }

    road_object[x].y=4; //start at bottom of tile
    //randomly place a tree on X axis
    //left side of road
    if(x==0)
    {
     if(road_x>=4)
      road_object[x].x=RAND_RANGE(0,road_x-4);
     else
      //road_object[x].x=(road_x-4)&31;
      road_object[x].x=(road_x-4);
    }
    //right side of road
    else
    {
      road_object[x].x=RAND_RANGE(road_x+road_lane+2,29);
      //no road side objects on beach
      if(game_level==1)
        road_object[x].object=0;        
    }
   }

   //normal tree
   if(road_object[x].object==1)
   {
    object_offset=(road_object[x].y-1)*4;
    //make sure we only draw objects on the map
    //if it is the top row, then we only draw the middle two tiles
    if(((road_object[x].x))<32 && road_object[x].y>1)
      g_TileMap[tile_offset+((road_object[x].x++))]=45+object_offset;
    else
      road_object[x].x++;
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=46+object_offset;
    else
      road_object[x].x++;
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=47+object_offset;
    else
      road_object[x].x++;
    //if it is the top row, then we only draw the middle two tiles
    if(((road_object[x].x))<32  && road_object[x].y>1)
      g_TileMap[tile_offset+((road_object[x].x))]=48+object_offset;
    road_object[x].y--;
    road_object[x].x-=3;
    //reset the counter for new object
    if(road_object[x].y<1)
    {
      //road_object[x].counter=10; 
      road_object[x].counter=RAND_RANGE(1,5);
      road_object[x].object=0;
    }
   }
   //palm tree
   else if(road_object[x].object==2)
   {
    object_offset=(road_object[x].y-1)*4;
    //make sure we only draw objects on the map
    //if it is the top row, then we only draw the middle two tiles
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=61+object_offset;
    else
      road_object[x].x++;
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=62+object_offset;
    else
      road_object[x].x++;
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=63+object_offset;
    else
      road_object[x].x++;
    //if it is the top row, then we only draw the middle two tiles
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x))]=64+object_offset;
    road_object[x].y--;
    road_object[x].x-=3;
    //reset the counter for new object
    if(road_object[x].y<1)
    {
      //road_object[x].counter=10; 
      road_object[x].counter=RAND_RANGE(1,5);
      road_object[x].object=0;
    }
   }

   //snow tree
   else if(road_object[x].object==3)
   {
    object_offset=(road_object[x].y-1)*4;
    //make sure we only draw objects on the map
    //if it is the top row, then we only draw the middle two tiles
    if(((road_object[x].x))<32 && road_object[x].y>1)
      g_TileMap[tile_offset+((road_object[x].x++))]=77+object_offset;
    else
      road_object[x].x++;
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=78+object_offset;
    else
      road_object[x].x++;
    if(((road_object[x].x))<32)
      g_TileMap[tile_offset+((road_object[x].x++))]=79+object_offset;
    else
      road_object[x].x++;
    //if it is the top row, then we only draw the middle two tiles
    if(((road_object[x].x))<32  && road_object[x].y>1)
      g_TileMap[tile_offset+((road_object[x].x))]=80+object_offset;
    road_object[x].y--;
    road_object[x].x-=3;
    //reset the counter for new object
    if(road_object[x].y<1)
    {
      //road_object[x].counter=10; 
      road_object[x].counter=RAND_RANGE(1,5);
      road_object[x].object=0;
    }
   }

  }
  else
   road_object[x].counter--;
 }
 //------------------------------
 //handle oil spot
  if(oil_timer>0)
  {
    if(car_dir!=3)
     oil_timer--;
  }
  else
  {
   //randomly deside when oil will come back up
   oil_timer=RAND_RANGE(min_oil,max_oil);
   //randomly place oil on road
   x=road_x+1+RAND_RANGE(0,road_lane-3);
   g_TileMap[tile_offset+x]=27; 
   g_TileMap[tile_offset+x+1]=28; 
  }
 //-----------------------------
 //handle finish line
 level_timer--;
 //end of the level, draw the finish line
 if(level_timer<2)
 {
  for(x=0;x<32;x++)
      g_TileMap[tile_offset+x]=10; 
  //new level
  if(level_timer==0)
  {
   game_seconds+=bonus_clock; //add time to clock
   draw_timer();
   level_timer=level_size;
   game_level++;
   if(game_level>2)
      game_level=0;
  }
 }
}

//---------------------------------------------------------
//animates car sprite (switches between the two sprite images)
void animate_racer()
{
	switch(car_dir)
	{
	  case 3: //car hit oil slick	
                 racer_animation_timer++;
                 if(racer_animation_timer>7)
                 {
		  racer_animation_timer=0;
                  racer_animation++;
                  racer_animation&=0x3; //only 4 animations
		  g_Sprites[0].PixelData = g_SpriteData+ (256*5);
		  switch(racer_animation)
		  {
			case 0: 
			  g_Sprites[0].PixelData = g_SpriteData;
			break;
			case 1: 
			  rotate_sprite_90((g_SpriteData),(g_SpriteData+(256*5)));
			break;
			case 2: 
			  rotate_sprite_180((g_SpriteData),(g_SpriteData+(256*5)));
			break;
			case 3: 
			  rotate_sprite_270((g_SpriteData),(g_SpriteData+(256*5)));
			break;

		  }
		 }
                 
	  break;
	  //normal animation
	  default:
                racer_animation_timer+=car_speed;
		if(racer_animation_timer>max_racer_animation_timer)
                {
		 //if(car_dir!=3) //lock the animation if we hit an oil slick 
                 racer_animation^=1;
                 racer_animation_timer-=max_racer_animation_timer;
                 //racer_animation=0;
		 g_Sprites[0].PixelData = g_SpriteData+ (racer_animation<<8);
                }
	  break;

	}
}
//---------------------------------------------------------
//copy a sprite
void copy_sprite(char *source_sprite, char *dest_sprite)
{
 int sprite1;
 for(sprite1=0;sprite1<256;sprite1++)
 {
  dest_sprite[sprite1]=source_sprite[sprite1];
 }
}

//---------------------------------------------------------
//copy a tile
void copy_tile(char *source_tile, char *dest_tile)
{
 char sprite1;
 for(sprite1=0;sprite1<64;sprite1++)
 {
  dest_tile[sprite1]=source_tile[sprite1];
 }
}

//---------------------------------------------------------
//rotates a sprite 90 degrees
void rotate_sprite_90(char *source_sprite, char *dest_sprite)
{
 int sprite1, sprite2, n=0,n2=15;
 sprite2=n2;
 for(sprite1=0;sprite1<256;sprite1++)
 {
  dest_sprite[sprite2]=source_sprite[sprite1];
  sprite2+=16;
  n++;
  if(n>15)
  {
   n=0;
   n2--;
   sprite2=n2;
  }
 }
}

//---------------------------------------------------------
//rotates a sprite 180 degrees
void rotate_sprite_180(char *source_sprite, char *dest_sprite)
{
 int sprite1, sprite2=255;
 for(sprite1=0;sprite1<256;sprite1++)
 {
  dest_sprite[sprite2]=source_sprite[sprite1];
  sprite2--;
 }
}

//---------------------------------------------------------
//rotates a sprite 270 degrees
void rotate_sprite_270(char *source_sprite, char *dest_sprite)
{
 int sprite1, sprite2, n=0,n2=240;
 sprite2=n2;
 for(sprite1=0;sprite1<256;sprite1++)
 {
  dest_sprite[sprite2]=source_sprite[sprite1];
  sprite2-=16;
  n++;
  if(n>15)
  {
   n=0;
   n2++;
   sprite2=n2;
  }
 }
}

//---------------------------------------------
//Draw Game Over
void draw_gameover()
{
//  TIMEHSCORGAVNW
// 01234567890123456
//           1111111
 
 unsigned const char s_gameover[]={10,11,3,4,0,8,12,4,9,0xFF};
 unsigned const char s_newhiscore[]={13,4,14,0,5,2,6,7,8,9,4,0,0xFF};
 unsigned char n;
 unsigned short int offset;

 //draw game over
 offset=(road_y+13)&0x1F;
 offset<<=5;
 offset+=11;
 for(n=0;n<10;n++)
 {
  if(s_gameover[n]==0xFF)
   break;
  else if(s_gameover[n]!=0)
   g_TileMap[offset]=s_gameover[n]+94;
  offset++;
 }
 
 //draw new hi score 
 if( (game_score==game_hiscore) && (game_score>0))
 {
  offset=(road_y+15)&0x1F;
  offset<<=5;
  offset+=10;
  for(n=0;n<13;n++)
  {
   if(s_newhiscore[n]==0xFF)
    break;
   else if(s_newhiscore[n]!=0)
    g_TileMap[offset]=s_newhiscore[n]+94;
   offset++;
  }
 }

}

//---------------------------------------------
//Draw HUD
void draw_hud()
{
//  TIMEHSCORGAVNW 
// 01234567890123456
//           1111111
 
 unsigned const char s_time[]={1,2,3,4,0};
 unsigned const char s_score[]={6,7,8,9,4,0};
 unsigned const char s_hiscore[]={5,2,6,7,8,9,4,0};
 unsigned char n;
 unsigned short int offset;
 //clear screen
 for(n=0;n<32*4;n++)
  g_TileMap2[n]=93;

 //draw time
 offset=(32*0)+13;
 for(n=0;n<9;n++)
 {
  if(s_time[n]!=0)
   g_TileMap2[offset++]=s_time[n]+94;
  else
   break;
 }
 //draw score
 offset=(32*1)+4;
 for(n=0;n<9;n++)
 {
  if(s_score[n]!=0)
   g_TileMap2[offset++]=s_score[n]+94;
  else
   break;
 }
 //draw a zero for score
 g_TileMap2[(32*2)+9]=111;

 //draw hiscore
 offset=(32*1)+20;
 for(n=0;n<9;n++)
 {
  if(s_hiscore[n]!=0)
   g_TileMap2[offset++]=s_hiscore[n]+94;
  else
   break;
 }
 //draw a zero for hiscore
 g_TileMap2[(32*2)+26]=111;

}

//---------------------------------------------
//draws game timer
void draw_timer()
{
 char temp_num[9];
 sprintf(temp_num,"%02u",game_seconds);
 print_number(14, 1, temp_num);
}

//---------------------------------------------
//draws score
void draw_hiscore()
{
 char temp_num[9];
 sprintf(temp_num,"%06u",game_hiscore);
 print_number(20, 2, temp_num);
}


//---------------------------------------------
//draws score
void draw_score()
{
 char temp_num[9];
 sprintf(temp_num,"%06u",game_score);
 print_number(3, 2, temp_num);
 if(game_score>game_hiscore)
 {
   game_hiscore=game_score;
   draw_hiscore();
 }
}

//---------------------------------------------
//get characters used in game 
void get_game_chars()
{
 int n, nx, ny, n2, offset;
 char t_char;
 //where we can start placing the font
 #define game_font_start 94
 #define ft_color 0xAE
 #define ft_bg 0x04
 //                  (space) T  I  M  E  H  S  C  O  R  G  A, V  N  W  D  Y
 const char char_set[]={32, 84,73,77,69,72,83,67,79,82,71,65,86,78,87,68,89};
//copy letters needed
//  TIMEHSCORGAVNWDY
// 01234567890123456
//           1111111

  for(n=0;n<17;n++)
  {
   offset=(8*char_set[n]);
   n2=((game_font_start+n)<<6);
   for(ny=0;ny<8;ny++)
   {
    t_char=font1bit[offset];
    //t_char=font_1bit[offset];
    offset++;
    for(nx=0;nx<8;nx++)
    {
     if(t_char & 0x80)
        g_TileData[n2]=ft_color;
     else
        g_TileData[n2]=ft_bg;
     t_char<<=1;
     n2++;
    }
   }
  }

  //copy numbers
  for(n=0;n<10;n++)
  {
   offset=(8*(48+n));
   //n2=((104+n)<<6);
   n2=((game_font_start+17+n)<<6);
   for(ny=0;ny<8;ny++)
   {
    t_char=font1bit[offset];
    //t_char=font_1bit[offset];
    offset++;
    for(nx=0;nx<8;nx++)
    {
     if(t_char & 0x80)
        g_TileData[n2]=ft_color;
     else
        g_TileData[n2]=ft_bg;
     t_char<<=1;
     n2++;
    }
   }
  }

}

//---------------------------------------------
void copy_font(int char_number, int char_data, unsigned char font_color, unsigned char font_bg)
{
#define font_offset 1 //where the font tiles starts
#define font_start 48 //where in the ascii chart the font starts
 //font_color, font_bg
 int n=0,n2=0, nx, ny, offset=8*48;
 unsigned char t_char;
 for(n=0;n<64;n++)
 {
  g_TileData[n2]=font_bg;
  n2++;
 }
 
 //numbers
 for(n=0;n<10;n++)
 {
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   //t_char=font_1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
 }
 //letters
 offset=8*65;
 for(n=0;n<26;n++)
 {
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   //t_char=font_1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
 }
  //period
  offset=8*46;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   //t_char=font_1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
  //solid block
  offset=8*0;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
  //corner peice 1
  offset=8*5;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
  //corner peice 2
  offset=8*6;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
  //corner peice 3
  offset=8*7;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }
  //corner peice 4
  offset=8*8;
  for(ny=0;ny<8;ny++)
  {
   t_char=font1bit[offset];
   offset++;
   for(nx=0;nx<8;nx++)
   {
    if(t_char & 0x80)
       g_TileData[n2]=font_color;
    else
       g_TileData[n2]=font_bg;
    t_char<<=1;
    n2++;
   }
  }

}
//---------------------------------------------
//prints a number
void print_number(int x, int y, char *string)
{
 int n, offset, screen_ctr;
 offset=111-48;
 screen_ctr=(y*SCREEN_TILE_MAP_WIDTH)+x;
 for(n=0;n<50;n++)
 {
   //select on the numbers
   if(string[n]>=48 && string[n]<58)
    g_TileMap2[screen_ctr++]=string[n]+offset; //draw digit
   else if(string[n]==0)
    return;
 }
}

//---------------------------------------------
//prints a string
void print_string(int x, int y, char *string)
{
 int n, offset, screen_ctr;
 offset=font_offset-font_start;
 screen_ctr=y*SCREEN_TILE_MAP_WIDTH+x;
 for(n=0;n<50;n++)
 {
   if(string[n]>=48 && string[n]<58)
   //make sure we are only printing characters that we have graphics for
   //if(string[n]>32 && string[n]<90)
    g_TileMap[screen_ctr]=string[n]+offset; //draw digit
   else if(string[n]>=65 && string[n]<90)
    g_TileMap[screen_ctr]=string[n]+offset-(65-58);
   else if(string[n]>=97 && string[n]<123)
    g_TileMap[screen_ctr]=string[n]+offset-(97-58);
   else if(string[n]==46)
    g_TileMap[screen_ctr]=offset+84;
   //solid block - enter in a $
   else if(string[n]==36)
    g_TileMap[screen_ctr]=offset+85;
   //corner peice 1 - enter in a ?
   else if(string[n]==63)
    g_TileMap[screen_ctr]=offset+86;
   //corner peice 2 - enter in a *
   else if(string[n]==42)
    g_TileMap[screen_ctr]=offset+87;
   //corner peice 3 - enter in a @
   else if(string[n]==64)
    g_TileMap[screen_ctr]=offset+88;
   //corner peice 4 - enter in a +
   else if(string[n]==43)
    g_TileMap[screen_ctr]=offset+89;

   else if(string[n]==0)
    return;
   screen_ctr++;
 }
}

void play_sound(unsigned char sound_number)
{
 //ranks sound by priority, higher number, then sound will play
 //0-no sound, 1-engine sound, 2-oil slick,3-car hit, 4-car crash
 //unsigned const char sound_prty[]={0,1,2,3,4};
 if(sound_number>current_sound)
 {
  current_sound=sound_number;
  engine_snd_timer=0;
 }
}

void sound_engine()
{
 //const int min_car_eng[]={ 70, 80, 90,100,110,120,130,140,150};
 //const int max_car_eng[]={ 90,100,110,120,130,140,150,160,170};
 const int min_car_eng[]={ 200,250,275,300,325,350,375,400,425};
 const int max_car_eng[]={ 350,400,425,450,475,500,525,550,575};
 const int car_hit_snd[]={ 200,300,400,375,350,350,350,300,250,200,0};
 //const int car_crash_snd[]={ 200,300,400,375,350,325,425,325,425,325,425,325,425,325,300,250,200,150,100,50,0};
 const int car_crash_snd[]={ 200,300,400,375,350,325,425,325,425,325,625,625,525,325,425,325,425,325,625,525,625,525,400,300,250,200,150,100,50,0};
 //car hit
 if(current_sound==3)
 {
  if(car_hit_snd[engine_snd_timer]>0)
  {
   SND_PlayTone(car_hit_snd[engine_snd_timer]);
   engine_snd_timer++;
  }
  else
  {
   current_sound=0;
   engine_snd_timer=0;
  }  
 }
 //sound of car crashing
 else if(current_sound==4)
 {
  if(car_hit_snd[engine_snd_timer]>0)
  {
   SND_PlayTone(car_crash_snd[engine_snd_timer]);
   engine_snd_timer++;
  }
  else
  {
   current_sound=0;
   engine_snd_timer=0;
  }  
 }

 else
 {
  //oil slick sound
  if(car_dir==3)
  {
   if(engine_snd_timer<1)
   {
      engine_sound^=1;
      if(engine_sound==0)
         SND_PlayTone(min_car_eng[car_speed]);
      else
         SND_PlayTone(max_car_eng[car_speed]);
       engine_snd_timer=5;
   }
   else
    engine_snd_timer--;
 
  }
  //engine sound
  else
  {
   if(engine_snd_timer<1)
   {
      engine_sound^=1;
      if(engine_sound==0)
         SND_PlayTone(min_car_eng[car_speed]);
      else
         SND_PlayTone(max_car_eng[car_speed]);
      engine_snd_timer=1;
   }
   else
    engine_snd_timer--;
  }
 }
}